"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const inquirer_1 = require("inquirer");
const installer_1 = require("./../engine/installer");
const generate_1 = require("./../generators/babel/generate");
const generate_2 = require("./../generators/jest/generate");
const generate_3 = require("./../generators/webpack/generate");
const flags_1 = require("./../utils/flags");
const format_1 = require("./../utils/format");
const detection_1 = require("./../utils/detection");
const pathing_1 = require("./../utils/pathing");
class Node extends command_1.Command {
    async run() {
        await Node.command(this.parse(Node));
    }
    static async command({ args, flags }) {
        switch (args.command) {
            case "module":
                await Node.moduleAction(args.arg, flags);
                return;
            default:
                const projectName = args.arg || ".";
                const options = await Node.initializePrompt(projectName);
                await Node.initializeAction(projectName, options, flags);
        }
    }
    static async initializePrompt(projectName) {
        if (process.env.IS_AUTOMATED === "true") {
            return {
                isJest: false,
                isNode: true,
                isTypescript: false
            };
        }
        const { options } = await inquirer_1.prompt({
            type: "checkbox",
            message: "Please select your options",
            name: "options",
            choices: [
                {
                    name: "Typescript",
                    value: "typescript"
                },
                {
                    name: "Jest",
                    value: "jest"
                }
            ]
        });
        return {
            isJest: options.indexOf("jest") > -1,
            isNode: true,
            isTypescript: options.indexOf("typescript") > -1
        };
    }
    static async initializeAction(path, opt, flags) {
        opt = Node.resolveFlagsToOptions(flags, opt);
        // Files
        let files = [];
        files.push(opt.isTypescript ? ["/packageJson/node/typescript", path] : ["/packageJson/node/javascript", path]);
        files.push(opt.isTypescript ? ["/node/init/typescript", path] : ["/node/init/javascript", path]);
        // Packages
        let devPackages = [];
        let prodPackages = [];
        if (opt.isTypescript) {
            devPackages.push("ts-node");
            devPackages.push("@types/node");
        }
        else {
            devPackages.push("@babel/node");
        }
        // Generators
        const webpackGen = generate_3.generateWebpack(opt, path);
        files = [...files, ...webpackGen.files];
        devPackages = [...devPackages, ...webpackGen.devPackages];
        prodPackages = [...prodPackages, ...webpackGen.prodPackages];
        if (!opt.isTypescript) {
            const babelGen = generate_1.generateBabel({}, path);
            files = [...files, ...babelGen.files];
            devPackages = [...devPackages, ...babelGen.devPackages];
            prodPackages = [...prodPackages, ...babelGen.prodPackages];
        }
        if (opt.isJest) {
            const jestGen = generate_2.generateJest(opt, path);
            files = [...files, ...jestGen.files];
            devPackages = [...devPackages, ...jestGen.devPackages];
            prodPackages = [...prodPackages, ...jestGen.prodPackages];
        }
        // Install
        await installer_1.install(files, devPackages, prodPackages, flags, path);
    }
    static async moduleAction(nameArg, flags) {
        // Validation
        if (!nameArg) {
            console.info("You must provide the name of the module");
            return;
        }
        const { name, dirPath } = pathing_1.resolvePath("modules", nameArg);
        // Files
        const stampVars = {
            name,
            extension: detection_1.isTypescriptProject(flags) ? "ts" : "js"
        };
        const files = [["/node/module", dirPath, stampVars]];
        // Install
        await installer_1.install(files, [], [], flags);
    }
}
exports.default = Node;
Node.docs = {
    description: "Sub cli for Node project generation.",
    descriptionShort: "Sub cli for Node project generation",
    commands: [
        {
            summary: "Initialize a node project.",
            summaryShort: "Initialize a node project",
            command: "tmpl node init"
        },
        {
            summary: "Initialize a node project.  " + "Force Typescript",
            summaryShort: "Initialize a node project",
            command: "tmpl node init -t"
        }
    ]
};
// OCLIF readme gen
Node.description = Node.docs.descriptionShort;
Node.examples = format_1.docCommandsToExamples(Node.docs.commands);
Node.flags = Object.assign(Object.assign(Object.assign({}, flags_1.installFlag), flags_1.javascriptFlags), flags_1.standardFlags);
Node.args = [{ name: "command" }, { name: "arg" }];
Node.resolveFlagsToOptions = (flags, opt) => {
    return Object.assign(Object.assign({}, opt), { isTypescript: opt.isTypescript || Boolean(flags.typescript) });
};
