"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const inquirer_1 = require("inquirer");
const installer_1 = require("./../engine/installer");
const generate_1 = require("./../generators/react/fc/generate");
const generate_2 = require("./../generators/jest/generate");
const generate_3 = require("./../generators/babel/generate");
const generate_4 = require("./../generators/webpack/generate");
const detection_1 = require("./../utils/detection");
const flags_1 = require("./../utils/flags");
const format_1 = require("./../utils/format");
const format_2 = require("./../utils/format");
class React extends command_1.Command {
    async run() {
        await React.command(this.parse(React));
    }
    static async command({ args, flags }) {
        switch (args.command) {
            case "init":
                await React.initialize(args.arg || ".", flags);
                break;
            case "context":
                await React.contextAction(args.arg, flags);
                break;
            case "fc":
                await React.fcAction(args.arg, flags, {
                    contextName: flags.contextName || "",
                    mobxName: flags.mobxName || ""
                });
                break;
            case "mst":
                await React.mstAction(args.arg, flags || "");
                break;
            case "test":
                await React.testAction(args.arg, flags);
                break;
            default:
                console.info("Missing / invalid subcommand");
        }
    }
    static async initialize(projectName, flags) {
        if (!projectName) {
            console.info("Project name was not provided");
            return;
        }
        const options = await React.initializePrompt(projectName);
        await React.initializeAction(projectName, options, flags);
    }
    static async initializePrompt(projectName) {
        if (process.env.IS_AUTOMATED === "true") {
            return {
                isJest: false,
                isSass: false,
                isReact: true,
                isTypescript: false
            };
        }
        const { options } = await inquirer_1.prompt({
            type: "checkbox",
            message: "Please select your options",
            name: "options",
            choices: [
                {
                    name: "Typescript",
                    value: "typescript"
                },
                {
                    name: "SCSS",
                    value: "sass"
                },
                {
                    name: "Jest",
                    value: "jest"
                }
            ]
        });
        return {
            isJest: options.indexOf("jest") > -1,
            isSass: options.indexOf("sass") > -1,
            isReact: true,
            isTypescript: options.indexOf("typescript") > -1
        };
    }
    static async initializeAction(projectName, opt, flags) {
        opt = React.resolveFlagsToOptions(flags, opt);
        // Files
        let files = [];
        files.push(["/packageJson/web", projectName]);
        files.push(["/react/init/core", projectName]);
        if (opt.isTypescript) {
            files.push(["/react/init/typescript/", projectName]);
        }
        else {
            files.push(["/react/init/javascript/", projectName]);
        }
        files.push(["/style/init", projectName]);
        files.push(["/style/reset", projectName]);
        // Packages
        let devPackages = ["@testing-library/react"];
        let prodPackages = ["react", "react-dom"];
        if (opt.isTypescript) {
            devPackages = [...devPackages, "@types/react", "@types/react-dom"];
        }
        // Generators
        const webpackGen = await generate_4.generateWebpack(opt, projectName);
        files = [...files, ...webpackGen.files];
        devPackages = [...devPackages, ...webpackGen.devPackages];
        prodPackages = [...prodPackages, ...webpackGen.prodPackages];
        if (!opt.isTypescript) {
            const babelGen = await generate_3.generateBabel(opt, projectName);
            files = [...files, ...babelGen.files];
            devPackages = [...devPackages, ...babelGen.devPackages];
            prodPackages = [...prodPackages, ...babelGen.prodPackages];
        }
        if (opt.isJest) {
            const jestGen = await generate_2.generateJest(opt, projectName);
            files = [...files, ...jestGen.files];
            devPackages = [...devPackages, ...jestGen.devPackages];
            prodPackages = [...prodPackages, ...jestGen.prodPackages];
        }
        // Install
        await installer_1.install(files, devPackages, prodPackages, flags, projectName);
    }
    static async contextAction(name, flags) {
        // Validation
        if (!name) {
            console.info("You must provide the name of the context");
            return;
        }
        // Files
        const stampVars = {
            name,
            nameProperCase: format_1.toProperCase(name)
        };
        const files = [
            detection_1.isTypescriptProject(flags)
                ? ["/react/context/typescript", "/src/stores", stampVars]
                : ["/react/context/javascript", "/src/stores", stampVars]
        ];
        // Install
        await installer_1.install(files, [], [], flags);
    }
    static async fcAction(name, flags, { contextName, mobxName }) {
        // Validation
        if (!name) {
            console.info("You must provide the name of the component");
            return;
        }
        // Files
        const stampVars = await generate_1.generateReactFcVars({
            name,
            contextName: contextName,
            mobxName: mobxName,
            isTypescript: detection_1.isTypescriptProject(flags)
        });
        const files = [["/react/fc", "./src/components", stampVars]];
        // Install
        await installer_1.install(files, [], [], flags);
    }
    static async mstAction(name, flags) {
        // Validation
        if (!name) {
            console.info("You must provide the name of the mobx state tree model");
            return;
        }
        // Files
        const stampVars = {
            name,
            nameProperCase: format_1.toProperCase(name)
        };
        const files = [
            detection_1.isTypescriptProject(flags)
                ? ["/react/mobxStateTree/typescript", "/src/stores", stampVars]
                : ["/react/mobxStateTree/javascript", "/src/stores", stampVars]
        ];
        // Packages
        let prodPackages = ["mobx", "mobx-react", "mobx-state-tree"];
        // Install
        await installer_1.install(files, [], prodPackages, flags);
    }
    static async testAction(name, flags) {
        // Validation
        if (!name) {
            console.info("You must provide the name of the component");
            return;
        }
        // Files
        const stampVars = await generate_1.generateReactFcVars({
            name,
            contextName: "",
            mobxName: "",
            isTypescript: detection_1.isTypescriptProject(flags)
        });
        const files = [["/react/test/", "./src/components", stampVars]];
        // Install
        await installer_1.install(files, [], [], flags);
    }
}
exports.default = React;
React.docs = {
    description: "Sub cli for React project init and generators",
    descriptionShort: "Sub cli for React project init and generators",
    commands: [
        {
            summary: "Create a new React project.",
            summaryShort: "Create a new React project",
            command: "tmpl react init"
        },
        {
            summary: "Create a functional component.",
            summaryShort: "Create a functional component",
            command: "tmpl react fc [name]"
        },
        {
            summary: "Create a functional component.  Force Typescript",
            summaryShort: "Create a functional component, force Typescript",
            command: "tmpl react fc [name] -t"
        },
        {
            summary: "Create a functional component with context connected.",
            summaryShort: "Create a functional component with context connected",
            command: "tmpl react fc [name] -c contextName"
        },
        {
            summary: "Create a functional component with mobx store connection.",
            summaryShort: "Create a functional component with mobx store connection",
            command: "tmpl react fc [name] -m mobxStoreName"
        },
        {
            summary: "Create a react data context under src/stores.",
            summaryShort: "Create a react data context under src/stores",
            command: "tmpl react context [name]"
        },
        {
            summary: "Create a mobx state tree model under src/stores.",
            summaryShort: "Create a mobx state tree model under src/stores",
            command: "tmpl react mst [name]"
        }
    ]
};
// OCLIF readme gen
React.description = React.docs.descriptionShort;
React.examples = format_2.docCommandsToExamples(React.docs.commands);
React.flags = Object.assign(Object.assign(Object.assign(Object.assign({}, flags_1.installFlag), flags_1.standardFlags), flags_1.javascriptFlags), { contextName: command_1.flags.string({ char: "c" }), mobxName: command_1.flags.string({ char: "m" }) });
React.args = [{ name: "command" }, { name: "arg" }];
React.resolveFlagsToOptions = (flags, opt) => {
    return Object.assign(Object.assign({}, opt), { isTypescript: opt.isTypescript || Boolean(flags.typescript) });
};
