"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const inquirer_1 = require("inquirer");
const installer_1 = require("./../engine/installer");
const generate_1 = require("./../generators/jest/generate");
const generate_2 = require("./../generators/babel/generate");
const generate_3 = require("./../generators/webpack/generate");
const format_1 = require("./../utils/format");
const detection_1 = require("./../utils/detection");
const flags_1 = require("./../utils/flags");
const format_2 = require("./../utils/format");
class Vue extends command_1.Command {
    constructor() {
        super(...arguments);
        this.resolveFlagsToOptions = (flags, opt) => {
            return Object.assign(Object.assign({}, opt), { isTypescript: Boolean(flags.typescript) });
        };
    }
    async run() {
        const { args, flags } = this.parse(Vue);
        switch (args.command) {
            case "init":
                await this.initialize(args.arg || ".", flags);
                break;
            case "component":
                await this.componentAction(args.arg, flags);
                break;
            default:
                this.log("No subcommand was provided");
        }
    }
    async initialize(projectName, flags) {
        if (projectName) {
            const options = await this.initializePrompt(projectName);
            await this.initializeAction(projectName, options, flags);
            return;
        }
        this.log("Project name was not provided");
    }
    async initializePrompt(projectName) {
        if (process.env.IS_AUTOMATED === "true") {
            return {
                isBabel: false,
                isJest: false,
                isSass: false,
                isReact: false,
                isTypescript: false,
                isVue: true
            };
        }
        const { options } = await inquirer_1.prompt({
            type: "checkbox",
            message: "Please select your options",
            name: "options",
            choices: [
                {
                    name: "Typescript",
                    value: "typescript"
                },
                {
                    name: "SCSS",
                    value: "sass"
                },
                {
                    name: "Jest",
                    value: "jest"
                }
            ]
        });
        return {
            isBabel: options.indexOf("typescript") === -1,
            isJest: options.indexOf("jest") > -1,
            isSass: options.indexOf("sass") > -1,
            isReact: false,
            isTypescript: options.indexOf("typescript") > -1,
            isVue: true
        };
    }
    async initializeAction(path, opt, flags) {
        opt = this.resolveFlagsToOptions(flags, opt);
        // Files
        let files = [];
        files.push(["/packageJson/web", path]);
        files.push(["/vue/init/core", path, { message: "{{message}}" }]);
        if (opt.isTypescript) {
            files.push(["/vue/init/typescript", path]);
        }
        else {
            files.push(["/vue/init/javascript", path]);
        }
        // Packages
        let prodPackages = ["vue"];
        let devPackages = [];
        // Generators
        const webpackGen = await generate_3.generateWebpack(opt, path);
        files = [...files, ...webpackGen.files];
        devPackages = [...devPackages, ...webpackGen.devPackages];
        prodPackages = [...prodPackages, ...webpackGen.prodPackages];
        if (!opt.isTypescript) {
            const babelGen = await generate_2.generateBabel(opt, path);
            files = [...files, ...babelGen.files];
            devPackages = [...devPackages, ...babelGen.devPackages];
            prodPackages = [...prodPackages, ...babelGen.prodPackages];
        }
        if (opt.isJest) {
            const jestGen = await generate_1.generateJest(opt, path);
            files = [...files, ...jestGen.files];
            devPackages = [...devPackages, ...jestGen.devPackages];
            prodPackages = [...prodPackages, ...jestGen.prodPackages];
        }
        // Install
        await installer_1.install(files, devPackages, prodPackages, flags, path);
    }
    async componentAction(componentName, flags) {
        // Validation
        if (!componentName) {
            this.log("You must provide the name of the component");
            return;
        }
        // Files
        const vars = {
            componentName,
            componentNameProperCase: format_1.toProperCase(componentName),
            componentNameKebabCase: format_1.toKebabCase(componentName)
        };
        const files = [
            detection_1.isTypescriptProject(flags)
                ? ["/vue/component/typescript", "./src/components/", vars]
                : ["/vue/component/javascript", "./src/components/", vars]
        ];
        // Install
        await installer_1.install(files, [], [], flags);
    }
}
exports.default = Vue;
Vue.docs = {
    description: "Sub cli for Vue project init and generators",
    descriptionShort: "Sub cli for Vue project init and generators",
    commands: [
        {
            summary: "Create a new Vue project.",
            summaryShort: "Create a new Vue project",
            command: "tmpl vue init"
        },
        {
            summary: "Create a vue component.",
            summaryShort: "Create a vue component",
            command: "tmpl vue component [name]"
        }
    ]
};
// OCLIF readme gen
Vue.description = Vue.docs.descriptionShort;
Vue.examples = format_2.docCommandsToExamples(Vue.docs.commands);
Vue.flags = Object.assign(Object.assign(Object.assign({}, flags_1.standardFlags), flags_1.installFlag), flags_1.javascriptFlags);
Vue.args = [{ name: "command" }, { name: "arg" }];
