"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const inquirer_1 = require("inquirer");
const installer_1 = require("./../engine/installer");
const generate_1 = require("./../generators/jest/generate");
const generate_2 = require("./../generators/babel/generate");
const generate_3 = require("./../generators/webpack/generate");
const flags_1 = require("./../utils/flags");
const format_1 = require("./../utils/format");
class Web extends command_1.Command {
    constructor() {
        super(...arguments);
        this.resolveFlagsToOptions = (flags, opt) => {
            return Object.assign(Object.assign({}, opt), { isTypescript: Boolean(flags.typescript) });
        };
    }
    async run() {
        const { args, flags } = this.parse(Web);
        let options = {
            isJest: false,
            isReact: false,
            isSass: false,
            isTypescript: false
        };
        switch (args.command) {
            case "init":
                options = await this.initializePrompt(".");
                await this.initializeAction(".", options, flags);
                break;
            case "component":
                await this.componentAction(args.arg, flags);
                break;
            default:
                this.log("No subcommand was provided");
        }
    }
    async initializePrompt(projectName) {
        if (process.env.IS_AUTOMATED === "true") {
            return {
                isJest: false,
                isReact: false,
                isSass: false,
                isTypescript: false
            };
        }
        const { options } = await inquirer_1.prompt({
            type: "checkbox",
            message: "Please select your options",
            name: "options",
            choices: [
                {
                    name: "Typescript",
                    value: "typescript"
                },
                {
                    name: "SCSS",
                    value: "sass"
                },
                {
                    name: "Jest",
                    value: "jest"
                }
            ]
        });
        return {
            isJest: options.indexOf("jest") > -1,
            isReact: false,
            isSass: options.indexOf("sass") > -1,
            isTypescript: options.indexOf("typescript") > -1
        };
    }
    async initializeAction(path, opt, flags) {
        opt = this.resolveFlagsToOptions(flags, opt);
        // Files
        let files = [];
        files.push(["/packageJson/web", path]);
        files.push(["/web/html", path]);
        files.push(["/style/init", path]);
        files.push(["/style/reset", path]);
        if (opt.isTypescript) {
            files.push(["/web/typescript", path]);
        }
        else {
            files.push(["/web/javascript", path]);
        }
        // Packages
        let devPackages = [];
        let prodPackages = [];
        // Generators
        const webpackGen = await generate_3.generateWebpack(opt, path);
        files = [...files, ...webpackGen.files];
        devPackages = [...devPackages, ...webpackGen.devPackages];
        prodPackages = [...prodPackages, ...webpackGen.prodPackages];
        if (!opt.isTypescript) {
            const babelGen = await generate_2.generateBabel(opt, path);
            files = [...files, ...babelGen.files];
            devPackages = [...devPackages, ...babelGen.devPackages];
            prodPackages = [...prodPackages, ...babelGen.prodPackages];
        }
        if (opt.isJest) {
            const jestGen = await generate_1.generateJest(opt, path);
            files = [...files, ...jestGen.files];
            devPackages = [...devPackages, ...jestGen.devPackages];
            prodPackages = [...prodPackages, ...jestGen.prodPackages];
        }
        // Install
        await installer_1.install(files, devPackages, prodPackages, flags, path);
    }
    async componentAction(name, flags) {
        let nameKebabCase = format_1.toKebabCase(name);
        // Validation
        if (!name) {
            this.log("You must provide the name of the component");
            return;
        }
        if (nameKebabCase.split("-").length < 2) {
            this.log("Web components must have a two part name, adding suffix");
            name += "Component";
            nameKebabCase += "-component";
        }
        // Files
        const stampVars = {
            name,
            nameKebabCase
        };
        const files = [["/web/component/javascript", "./src/elements", stampVars]];
        // Packages
        const prodPackages = ["lit-html"];
        // Install
        await installer_1.install(files, [], prodPackages, flags);
    }
}
exports.default = Web;
Web.docs = {
    description: "Sub cli for vanilla web projects",
    descriptionShort: "Sub cli for vanilla web projects",
    commands: [
        {
            summary: "Inits new web project.",
            summaryShort: "Inits new web project",
            command: "tmpl web init"
        },
        {
            summary: "Creates a web component.",
            summaryShort: "Creates a web component",
            command: "tmpl web component [name]"
        }
    ]
};
// OCLIF readme gen
Web.description = Web.docs.descriptionShort;
Web.examples = format_1.docCommandsToExamples(Web.docs.commands);
Web.flags = Object.assign(Object.assign(Object.assign({}, flags_1.standardFlags), flags_1.installFlag), flags_1.javascriptFlags);
Web.args = [{ name: "command" }, { name: "arg" }];
